function [fullfilename] = EPSATbuildfullfilename(ncfilesset, Annee, Mois, Jour)

%EPSATBUILDFULLFILENAME permet d'obtenir le chemin d'acces a un fichier
%NetCDF, de donnees de precipitations estimes d'apres l'algorithme EPSAT,
%pour la date desiree.

%
%+
%
% =========================
% EPSATbuildfullfilename.m
% =========================
%
% .. function:: EPSATbuildfullfilename(ncfilesset, Annee, Mois, Jour)
%
% DESCRIPTION
% ===========
%
% - Donnees d'entrees :
%
%   * ncfilesset (en caracteres) : Type de fichier NetCDF que l'on cherche,
%     'normal' ou 'extracted'.
%   * Annee (en caracteres) : Annee (AAAA) du jour d'etude.
%   * Mois (en caracteres) : Numero du mois (MM) du jour d'etude.
%   * Jour (en caracteres) : Numero du Jour d'etude (JJ).
%
% - Donnees de sorties :
%
%   * fullfilename : Chemin d'acces au fichier pour la date desiree.
%
% Cette fonction permet d'obtenir le chemin d'acces a un fichier NetCDF,
% de donnees de precipitations estimes d'apres l'algorithme EPSAT, pour la
% date desiree.
%
% EXAMPLES
% ========
%
% Voir la 'demo'.
%
% SEE ALSO
% ========
%
% :ref:`data_epsat`
%
% :ref:`varamma_startup.m`
%
% :func:`EPSATread`
%
% TODO
% ====
%
% add demo extracted
%
% test if file exists and handling error
%
% version in filname is not always v3.1-02c
%
% EVOLUTIONS
% ==========
%
% $Id$
%
% $URL: svn+ssh://pinsard@forge.ipsl.jussieu.fr/ipsl/forge/projets/varamma/svn/trunk/src/EPSATbuildfullfilename.m $
%
% - fplod 20110804T112630Z aedon.locean-ipsl.upmc.fr (Darwin)
%
%   * date is a reserved word : replaced by date_file
%
% - jaclod 2011-07-26
%
%   * Revision de la documentation et des commentaires.
%
% - jaclod 2011-07-21
%
%   * Adaptation pour les fichiers 'extracted'.
%
% - jaclod 2011-07-13
%
%   * Creation
%
%-
% Appel de la variable global 'PROJECT_ID' (Il faut auparavant avoir
% execute 'varamma_startup').
global PROJECT_ID

usage='[fullfilename] = EPSATbuildfullfilename(ncfilesset, Annee, Mois, Jour)';
if nargin~=nargin(mfilename)
   disp(['Incorrect number of input arguments ' num2str(nargin)]);
   error(usage)
end
%
if nargout ~= nargout(mfilename)
   disp(['Incorrect number of output arguments ' num2str(nargout)]);
   error(usage);
end
%
% Creation d'une variable comportant la date nous permettant de
% selectionner le fichier en question.
date_file = ([Annee Mois Jour]);

% Le repertoire ou se trouve le fichier, ainsi que son nom, different selon
% qu'il soit 'normal' (fichier original) ou 'extracted' (fichier avec
% extraction des donnees spatiales).
switch ncfilesset

    case 'normal'

        % Repertoire contenant le fichier souhaite.
        repertoire = [ PROJECT_ID 'EPSAT/' Annee '/' Mois '/' ];

        % Creation du chemin complet.
        fullfilename = [repertoire 'surf-rr_epsat-sg_multi-sat_010d_30min_' date_file '_v3.1-02c.nc'];

    case 'extracted'

         % Repertoire contenant le fichier souhaite.
        repertoire = [PROJECT_ID 'EPSAT/extracted/' Annee '/' Mois '/' ];

        % Creation du chemin complet.
        fullfilename = [repertoire 'extracted-surf-rr_epsat-sg_multi-sat_010d_30min_' date_file '_v3.1-02c.nc'];

    otherwise

        error('La variable ncfilesset doit etre ''normal'' ou ''extracted''.');

end

end

%!demo
%! varamma_startup;
%! more off;
%! Annee = '2006';
%! Mois = '08';
%! Jour = '01';
%! ncfilesset = 'normal';
%! [fullfilename] = EPSATbuildfullfilename(ncfilesset, Annee, Mois, Jour)
