 ; docformat = 'rst'
;+
;
; ============
; cresamsu.pro
; ============
;
; .. function: cresamsu(numch, yyyyb, mmb, ddb, yyyye, mme, dde,
; lonmin, lonmax, latmin, latmax, temp, rtemp, rhdeg)
;
; DESCRIPTION
; ===========
;
; programme d'interpolation avec algorithme de Cressmann
;
;; grille de sortie en long / lat
; recoit en entree les jours de l'annee jdeb et jfin
;correspondant au debut et la fin du mois
;
; adaptation aux donnees Tb AMSU lues avec la nouvelle chane de lecture
; extract_amsu.pro
; correct_nadir.pro
;
; but: faire des fichiers "grilles" pour analyse statistique avec
; interpolation spatiale et temporelle
;
; .. only:: man
;
;    Figure is visible on PDF and HTML documents only.
;
; .. only:: html or latex
;
;     .. graphviz::
;
;        digraph cresamsu {
;           amsu_t2 [shape=ellipse,
;               fontname=Courier,
;               label="${PROJECT_ID}/yyyy/mm/numch_yyyymmdd_geomin_geomax.dat"];
;           amsu_t5 [shape=ellipse,
;               fontname=Courier,
;               label="${PROJECT_OD}/AMSU/numch_temprtrtemprhrhdeg_yyyymmdd_yyyymmdd_geomin_geomax.nc"];
;           cresamsu [shape=box,
;           fontname=Courier,
;           color=blue,
;           URL="http://forge.ipsl.jussieu.fr/varamma/browser/trunk/src/cresamsu.pro",
;           label="${PROJECT}/src/cresamsua.pro"];
;
;           {amsu_t2} -> {cresamsua} -> {amsu_t5};
;       }
;
; EXAMPLES
; ========
;
; ::
;
;    numch='a5'
;    yyyyb=2006L
;    yyyye=2006L
;    mmb=8
;    mme=8
;    ddb=1
;    dde=3
;    lonmin=-25.
;    lonmax=25.
;    latmin=-5.
;    latmax=20.
;    temp='m'
;    rtemp=0.5
;    rhdeg=1.
;    result = cresamsu(numch, yyyyb, mmb, ddb $
;            , yyyye, mme, dde $
;            , lonmin, lonmax, latmin, latmax $
;            , temp, rtemp, rhdeg);
;
; SEE ALSO
; ========
;
;  Previous step :ref:`extract_amsu.pro` (:ref:`traite_amsuab.sh`)
; :ref:`traite_amsuab.sh`
;
; Use :func:`file_amsu_t2_to_mem
; :func:`idl_amsu_netcdf`
;
; TODO
; ====
;
;
; fix pb with fov_int2 and fov_int3
;
; check last line for uncomment endfor (not sure of the use of this line)
;
; aligner lal/lon de l'exemple sur les autres modules amsu
;
; 
; coding rules
;
; new filename convention
;
; get rid of idldoc (fplod)
;
; EVOLUTIONS
; ==========
;
;  lelod, fin mai 2012
;  ajout de correction supplementaire pour ameliorer les champs
; lecture et application de la correction avant interpolation
;  $Id: 
;
;  $URL$
;
; - fplod 20120430
;
;   * pro -> function
;   * idl_amsu_netcdf is a function
;   * add digraph
;
; - lelod 20120410 puis 20120503
;
;   * parametres de la grille dependant des rayons de cresmann
;   * revisite de la grille temporelle
;   * choix du rayon de Cressmann en degres et pas en km
;   * ajout d'un parametre temp (= "m" ou "e") pour choix de la plage
;   * ajout en parametres des rayons de Cressmann rtemp et rhdeg
;     horaire d'interpolation temporelle
;
; - fplod 20120323
;
;   * homogenize examples to traite_amsuab.sh
;   * handling no existence of one file
;   * handling no existence of all files
;
; - pinsard 20111216T172940Z
;
;   * start to fix usage of result of file_amsu_t2_to_mem
;
; - fplod 20111216T164041Z cratos (Linux)
;
;   * change the name of the procedure to avoid ambiguity
;   * fix file_amsu_t2_to_mem call
;   * start to forgot idldoc
;   * fix for loop syntax error
;
; - lelod 20121215
;
;   * reprise du programme apres refonte de la chaine (extract_amsua):
;     lecture nouveau format de fichier (structure)
;     elimination des tentatives precedentes
;     ajout du mask dans l'appel a idl_amsu_ncdf
;     introduire en parametres les choix pour le fichier ncdf: grille,
;     rayons de Cressamann, selection de la portion du jour utilisee dans
;     l'interpolation
;
; - lelod 20111024
;
;   * prise en compte des modifications de lecture et correction nadir
;     (correct_nadir.pro):
;     possibilite de traiter amsua et amsub ou amsub seul
;     utilisation de pixelsize.pro pour boucher les trous entres lignes de
;     pixels en bordure de fauchee de facon correcte
;
; - fplod 20110505T123230Z cratos.locean-ipsl.upmc.fr (Linux)
;
;   * minimal header
;
; - le 20110504
;
;-

function cresamsu $
   , numch $
   , yyyyb $
   , mmb $
   , ddb $
   , yyyye $
   , mme $
   , dde $
   , lonmin $
   , lonmax $
   , latmin $
   , latmax $
   , temp $
   , rtemp $
   , rhdeg

compile_opt idl2, strictarrsubs
@cm_project
;
s = size(SCOPE_TRACEBACK(/STRUCTURE),/DIMENSION)
routine = (SCOPE_TRACEBACK(/STRUCTURE))[s-1].Routine
;
key_performance=0
if key_performance EQ 1 THEN BEGIN
    time1 = SYSTIME(1)
ENDIF
;
;
; Return to caller if errors
ON_ERROR, 2
result = -1
;
usage = 'result = cresamsu(numch, yyyyb, mmb, ddb, yyyye, mme, dde, lonmin, lonmax, latmin, latmax, temp, rtemp, rhdeg)'
;
nparam = N_PARAMS()
IF (nparam NE 14) THEN BEGIN
    ras = report(['Incorrect number of arguments.' $
          + '!C' $
          + 'Usage : ' + usage])
    return, result
ENDIF
;
tbmin=100
tbmax=350
; est il utile d'avoir encore les seuils sur les Tbs a cette
; etape?

; rayons de cressmann et grille de sortie
;----------------------------------------
;rayon temporel fourni en parametre rtemp
;calcul sur orbites du matin (temp='m'), du soir (temp='e') ou
;indifferemment (temp='d')
rtemphor=rtemp*24. ; en heures

; caracteristiques de la grille de sortie
; par defaut: orbites du matin
pastemp=1
hdeb=0
hfin=12                               ; on ne cherchera pas de donnees eloignees de plus de 6 heures
heurdebut=6 ; grille centree sur 6h du matin
if temp eq 'e' then begin
   heurdebut=18
   hdeb=12
   hfin=24
endif
if temp eq 'd' then begin
   hdeb=0
   hfin=24                      ; on prend toutes les orbites de la journee (regule par rtemp)
   if rtemp le 0.5 then begin
      pastemp=0.5
   endif else begin
      pastemp=1
   endelse
endif
pasthor=pastemp*24.

; debut et fin de la grille temporelle, a partir des fichiers d'entree
; en j juliens : tient compte du rayon de Cressmann temporel (rtemp)
j0=julday(mmb,ddb,yyyyb,0)
j1=julday(mme,dde,yyyye,0)
if (rtemp le heurdebut/24.) then begin
   jdeb=j0+heurdebut/24.
endif else begin
   if rtemp lt 2 then begin
      jdeb=j0+heurdebut/24.+pastemp
   endif else begin
      jdeb=j0+heurdebut/24.+fix(rtemp)
   endelse
endelse
if (rtemp le 1-heurdebut/24.) then begin
   jfin=j1+heurdebut/24.
endif else begin
   if rtemp lt 2 then begin
      jfin=j1+heurdebut/24.-pastemp
   endif else begin
      jfin=j1+heurdebut/24.-fix(rtemp)
   endelse
endelse
hrdeb=jdeb*24
hrfin=jfin*24
msg = report(['iii : ' + routine + ' : debut et fin = ' + string(jdeb) + ' ' + string(jfin)])

jmax=fix((hrfin-hrdeb)/pasthor) ; valeurs de 0 a jmax
nbjour=jmax+1


; conversions degres / radians et rayon en km
vbidon=-999.
coef=!pi/180.
rterre=6400.
rhor=rhdeg*(rterre*!pi)/180. ; en km
r2=rhor*rhor
; resolution spatiale et temporelle de la grille de sortie
pash=rhdeg/2. ; en degres
; grille des donnees en sortie
lo1=lonmin+rhdeg
lo2=lonmax-rhdeg
la1=latmin+rhdeg
la2=latmax-rhdeg
print,"bornes de la grille de sortie",lo1,lo2,la1,la2,jdeb,jfin
kmax=fix((lo2-lo1)/pash) ; pour valeurs de 0 a kmax
nblon=kmax+1
lmax=fix((la2-la1)/pash) ; valeurs de 0 a lmax
nblat=lmax+1
msg = report(['iii : ' + routine + ' : nb pas de temps = ' + string(nbjour)])
msg = report(['iii : ' + routine + ' : nb lat = ' + string(nblat)])
msg = report(['iii : ' + routine + ' : nb lon = ' + string(nblon)])
msg = report(['iii : ' + routine + ' : rayon temporel en heures' + string(rtemphor)])


;____________________________________
; pour le fichier netcdf
;------------------------------------
; coordonnees des points de la grille en sortie
los=coef*(indgen(nblon)*pash+lo1) ; en radians
las=coef*(indgen(nblat)*pash+la1) ; idem
xxlon=(indgen(nblon)*pash+lo1) ; en degres
yylat=(indgen(nblat)*pash+la1) ; en degres
jjours=indgen(nbjour)*pastemp+jdeb
lo2=max(xxlon)
la2=max(yylat)
hrfin=max(jjours)*24.
msg = report(['iii : ' + routine + ' : pas de temps du fichier ncdf en jours decimaux= ' + string(jjours)])
msg = report(['iii : ' + routine + ' : derniers points de la grille (lon) = ' + string(max(xxlon))])
msg = report(['iii : ' + routine + ' : derniers points de la grille (lat) = ' + string(max(yylat))])
msg = report(['iii : ' + routine + ' : derniers points de la grille (temps) = ' + string(max(hrfin))]);-----------------------------------
t=indgen(nbjour)*pasthor+hrdeb ; en heures
;print,'t',t

; initialisation des tableaux de sortie (s valeurs, z poids, kont nb points)
stb1=fltarr(nblon,nblat,nbjour)
kont1=intarr(nblon,nblat,nbjour)
z1=fltarr(nblon,nblat,nbjour)
tb1fin=fltarr(nblon,nblat,nbjour)
; lecture des donnees et calcul des points interpoles
; lecture du fichier ajuste au nadir
; lecture sequentielle des donnees d'entree et ajout aux points de grille
; concernes
prefix=numch
cont1=110L
cont2=120L   ; ?????
look = 'filename'
geomin = geolocation_to_string_idl(lonmin, latmin, look,1)
geomax = geolocation_to_string_idl(lonmax, latmax, look,1)

; correction supplementaire si necessaire
correcbis='non'
if correcbis eq 'oui' then begin
    fovmax=46
    openr,1,project_id_env+'AMSU/AMSU'+numch+'_correcbis_t_JJAS2009.dat'
    moytb=fltarr(fovmax)
    readf,1,moytb,format='(46(f6.2,1X))'
    close,1
    moytbliss=smooth(moytb,5)
    correction_t=moytbliss-mean(moytbliss[21:26])
    openr,2,project_id_env+'AMSU/AMSU'+numch+'_correcbis_m_JJAS2009.dat'
    moytb=fltarr(fovmax)
    readf,2,moytb,format='(46(f6.2,1X))'
    close,2
    close,/all
    moytbliss=smooth(moytb,5)
    correction_m=moytbliss-mean(moytbliss[22:24])
endif


testfilename=''
nbfile = 0L
; lecture des fichiers
;ddmb=ddb
;ddme=dde
;mmyb=mmb
;mmye=mme
;for yyyy=yyyyb,yyyye do begin
;   if (yyyye ne yyyyb) then begin
;      mmyb=1
;      mmye=12
;      if yyyy eq yyyyb then mmyb=mmb
;      if yyyy eq yyyye then mmye=mme
;   endif
;   print,"traitement annee ",yyyy,'mois ',mmyb,'a ',mmye
;   for mm=mmyb,mmye do begin
;      if (mme ne mmb) then begin
;         ddmb=1
;         ddme=31
;         if mm eq mmb then ddmb=ddb
;         if mm eq mme then ddme=dde
;     endif
;      print,"mois ",mm,"jours ",ddmb,"a ",ddme
;      for dd=ddmb,ddme do begin
dateb=julday(mmb,ddb,yyyyb)
datee=julday(mme,dde,yyyye)
print,'dates debut et fin en jours juliens',dateb,datee
for jd=dateb,datee do begin
   caldat,jd, mm,dd,yyyy
   print, 'iii : traitement du jour ',yyyy,mm,dd
         ; lecture du fichier journalier
   result=file_amsu_t2_to_mem( yyyy, mm, dd, numch $
                               , lonmin, lonmax, latmin, latmax $
                               , testfilename)
   if (size(result,/TYPE) EQ 3) THEN BEGIN
      goto, nofile
   ENDIF
   desc=result.data.desc
   hour=result.data.hour
   fov=result.data.fov
   lon=result.data.lon
   lat=result.data.lat
   mask=result.data.landseamask
   tbinit=result.data.tb
                                ; decodage du nb de points (nn)
   nn=n_elements(tbinit)
; egalement aussi des bornes en lon / lat (a retirer des parametres d'entree)++

; boucle sur les points du fichier
   tb1=tbinit
   if correcbis eq "oui" then begin
      print,"ATTENTION: correction supplementaire activee"
; application de la correction bis
      tb1=fltarr(nn)
      print,'fauchee',min(fov),max(fov)
      for ifov=min(fov)+1,fovmax-1 do begin
         ind=where(fov eq ifov and mask eq 1,nbfov)
         if nbfov ne 0 then begin
            tb1[ind]=tbinit[ind] -correction_t[ifov]
         endif
      endfor
      ind=where(fov eq 1,smallfov)
      if smallfov ne 0 then begin
         tb1[ind]=tbinit[ind] -correction_t[0]
      endif
      ind=where(fov ge fovmax,bigfov)
      if bigfov ne 0 then begin
         tb1[ind]=tbinit[ind] -correction_t[fovmax-1]
      endif
      for ifov=min(fov)+1,fovmax-1 do begin
         ind=where(fov eq ifov and mask eq 0,nbfov)
         if nbfov ne 0 then begin
            tb1[ind]=tbinit[ind]-correction_m[ifov]
         endif
      endfor
      ind=where(fov eq 1,smallfov)
      if smallfov ne 0 then begin
         tb1[ind]=tbinit[ind] -correction_m[0]
      endif
      ind=where(fov ge fovmax,bigfov)
      if bigfov ne 0 then begin
         tb1[ind]=tbinit[ind] -correction_m[fovmax-1]
      endif
   endif
   
   for i=0,nn-1 do begin
; recup de tb1,hour,lon,lat
      if (tb1[i] ge tbmin and tb1[i] le tbmax) then begin
;if  ((tb1 lt tbmin) or (tb1 gt tbmax)) then goto,suite0 ;; on ne s interesse pas aux donnees aberrantes
; a remplacer par if tb1 ne Nan then begin
;if (cont lt 10) then print,lon,lat,fov,tb1,hour
; test sur le debut et la fin des donnes utiles tenant compte du rayon temporel
         jour=julday(mm,dd,yyyy,0)*24.+hour[i] ; en heures decimales
                                ;if (cont lt 10) then print,mm,dd,yyyy,'jour julien lu (en heure)',jour
         if ((jour ge hrdeb-rtemphor) and (jour le hrfin+rtemphor) and (hour[i] le hfin) and (hour[i] ge hdeb) ) then begin
                                ; print,'on cherche les donnees dans la fenetre temporelle'
                                ; utilisation seulement des orbites du matin - projet heat low
                                ; grilles temporelle et spatiale utile
            jdp=fix((jour-hrdeb-rtemphor)/pasthor)>0
            jfn=fix((jour-hrdeb+rtemphor)/pasthor)+1<jmax
                                ;print,'grille utile temporelle',jdp,jfn
            kdp=fix((lon[i]-lo1-rhdeg)/pash)>0
            kfn=fix((lon[i]-lo1+rhdeg)/pash)+1<kmax
            ldp=fix((lat[i]-la1-rhdeg)/pash)>0
            lfn=fix((lat[i]-la1+rhdeg)/pash)+1<lmax
                                ;print,'grille spatiale utile', kdp,kfn,ldp,lfn
                                ; calcul de la fonction de Cressman
            yy=lat[i]*coef
            xx=lon[i]*coef
            coyy=cos(yy)
            coxx=cos(xx)
            sixx=sin(xx)
            siyy=sin(yy)
                                ; boucle pour le calcul des contributions aux points de la grille
                                ; (los, las, t)
            for nj=jdp,jfn do begin
               dij=(jour-t[nj])/rtemphor ; normalisee par le rayon de Cressmann
               disj=dij*dij
               for l=ldp,lfn do begin
                  coco=coyy*cos(las[l])
                  sisi=siyy*sin(las[l])
                  for k=kdp,kfn do begin
                     xxs=los[k]
                     ang=coco*(coxx*cos(xxs)+sixx*sin(xxs))+sisi
                     vv=ang
                     ac=acos(vv)
                     dist=rterre*rterre*ac*ac/r2
                     if (dist lt 1. and disj lt 1.) then begin
                        rnorm=2.
                        poids=(rnorm-dist-disj)/(rnorm+dist+disj)
                        stb1[k,l,nj]=stb1[k,l,nj]+tb1[i]*poids
                        z1[k,l,nj]=z1[k,l,nj]+poids
                        kont1[k,l,nj]=kont1[k,l,nj]+1
                                ; print,kont1[k,l,nj]
                     endif
                  endfor
               endfor
            endfor
         endif
      endif
            ; fin test sur donnees valides (inutile normalement)
   endfor
         ; fin lecture donnees fichier journalier
   nbfile = nbfile + 1L
   goto, nextday
nofile:
   print, 'www : no data to read for this day'
   goto, nextday

nextday:
   IF key_performance EQ 1 THEN BEGIN
      msg = report(['ppp : ' + routine + ' : dure intermediaire au passage au fichier suivant' $
                    + string(SYSTIME(1)-time1,format='(F12.6)')])

   ENDIF
endfor

; fin boucle sur le temps


if nbfile GT 0 THEN BEGIN
    good=where (kont1 ge 1,nb)
    msg = report(['iii : ' + routine + ' : points valides = ' + string(nb)])
    if (nb ne 0) then tb1fin[good]=stb1[good]/z1[good]
    bad=where (kont1 eq 0,nbad)
    print,'bad',nbad
    if (nbad ne 0) then tb1fin[bad]=!values.f_Nan
                                    ; print,tb1fin
    msg = report(['iii : ' + routine + ' : extremes du compteur = ' + string(min(kont1)) + ' ' + string(max(kont1))])

    result=idl_amsu_netcdf(numch, temp,rtemp,rhdeg, tb1fin, yylat, xxlon, jjours)
    result_type=size(result,/type)
    IF (result_type EQ 3) THEN BEGIN
       IF (result NE 0) THEN BEGIN
           msg = report(['eee : ' + routine + ' : pb idl_amsu_netcdf'])
           ;return, result
           STOP
       ENDIF
    ENDIF
ENDIF  ELSE BEGIN
      msg = report(['www : ' + routine + 'no files at all'])
    goto, realend
 ENDELSE

realend:
IF key_performance EQ 1 THEN BEGIN
    msg = report(['ppp : ' + routine $
         + ' : dure totale = ' $
         + string(SYSTIME(1)-time1, format='(a, 1x, F12.6)')])
ENDIF

msg = report(['iii : ' + routine + ': fin'])
	
result = 0
return, result	
end
