;+
;
; =======================
; file_amsu_t2_to_mem.pro
; =======================
;
; .. function:: file_amsu_t2_to_mem(yyyy, mm, dd, numch, lonmin, lonmax, latmin, latmax, testfilename)
;
; DESCRIPTION
; ===========
;
; Read an amsu_t2 file and store data and header in structure
;
; This type of file has been produced by :ref:`extract_amsua.pro`
;
; If **testfilename** is not empty, this filename will be written, otherwise
; amsu_t2 terminology will be applied.
;
; :returns: data structure +todo+ details or -1 if error
; :rtype: structure +todo+ details
;
; EXAMPLES
; ========
;
; Realistic example with VARAMMA file AMSU-A chanel 5
; :file:`${PROJECT_ID}/AMSU/2006/08/a5_20060813_025w05s_025e20n.dat`::
;
;   numch = 'a5'
;   yyyy = 2006L
;   mm = 8
;   dd = 13
;   lonmin = -25.
;   lonmax = 25.
;   latmin = -5.
;   latmax = 20.
;   testfilename = ''
;   result = file_amsu_t2_to_mem(yyyy, mm, dd, numch, lonmin, lonmax, latmin, latmax, testfilename)
;
; impression de controle : structure et 1re ligne de donnes::
;
;   help, result, /structure
;   print, result.data[0]
;
; SEE ALSO
; ========
;
; :ref:`extract_amsua.pro`
;
; :func:`mem_to_file_amsu_t2`
;
; use :func:`geolocation_to_string_idl`
;
; TODO
; ====
;
; ajouter de # dans les lignes d'entete
;
;  intgrer dansla chaine de traitement amsu
;
; check args
;
; ecrire le module de dessin
;
; EVOLUTIONS
; ==========
;
; $Id$
;
; $URL$
;
; - fplod 20120323
;
;   * handling file error 
;
; - fplod 20111213T143503Z aedon.locean-ipsl.upmc.fr (Darwin)
;
;   * geobox in filename
;
; - fplod 20111213T094118Z cratos (Linux)
;
;   * ajout de resol dans l'en tete
;   * chgt de type des coordonnes : 0L vers 0.0L
;
; - fplod 20111209T150657Z cratos (Linux)
;
;   * suppression des parametres de zone gographique
;   * ajout de nbpix dans la premire ligne de header
;   * ajout d'une seconde ligne d'entete avec les coordonnes de la zone

; - fplod 20111208T110651Z cratos (Linux)
;
;   * changement de format et de terminologie
;
; - fplod 20111206T161848Z aedon.locean-ipsl.upmc.fr (Darwin)
;
;   * make it ok with files created by extract_amsuab examples
;   * better variable names inside the structure thank to
;     :ref:`correct_nadir_amsu.pro`
;
; - fplod 20111205T094831Z aedon.locean-ipsl.upmc.fr (Darwin)
;
;   * creation thanks to http://www.idlcoyote.com/tips/ascii_column_data.html
;
;     semble ok pour le fichier de test amsub seul
;
;-
FUNCTION file_amsu_t2_to_mem, yyyy, mm, dd, numch $
                            , lonmin, lonmax, latmin, latmax $
                            , testfilename
;
compile_opt idl2, strictarrsubs
;
@cm_project
;
; Return to caller if errors
ON_ERROR, 2
;
result = -1
;
usage = 'result=file_amsu_t2_to_mem(yyyy, mm, dd, numch, testfilename)'
nparam = N_PARAMS()
IF (nparam NE 9) THEN BEGIN
   ras = report(['Incorrect number of arguments.' $
         + '!C' $
         + 'Usage : ' + usage])
   return, result
ENDIF
;
if (testfilename eq '') then begin
    look = 'filename'
    scale = 1.
    geomin = geolocation_to_string_idl(lonmin, latmin, look, scale)
    geomax = geolocation_to_string_idl(lonmax, latmax, look, scale)
    fullfilename = project_id_env+ 'AMSU/' $
            + string(yyyy,format='(I4.4)') + '/' $
            + string(mm,format='(I2.2)')   + '/' $
            + numch + '_' $
            + string(yyyy,format='(I4.4)') $
            + string(mm,format='(I2.2)') $
            + string(dd,format='(I2.2)') + '_' $
            + geomin + '_' $
            + geomax $
            + '.dat'
endif else begin
    fullfilename = testfilename
endelse
;
IF (FILE_TEST(fullfilename) EQ 0) THEN BEGIN
   msg = 'eee : the file ' + fullfilename  + ' not found'
   ras = report(msg)
   return, result
ENDIF

;
; build header format string
format_header1 = '(I4.4,I2.2,I2.2,1X,A2,1X,I3.3,1X,F6.3)'
format_header2 = '(1x,f8.3,1x,f8.3,1x,f8.3,1x,f8.3)'
;
; build data format string
format_data = '(i2,1x,f8.4,i3,1x,f8.3,1x,f8.3,1x,I1,1x,f8.3)'
;
header1= { yyyy : 0L $
	            ,  mm : 0L $
	            ,  dd : 0L $
		    ,  numch : 'X' $
		    ,  nbpix : 0L $
		    ,  resol : 0.0 $
		     }
header2= { lonmin: 0.0 $
	              , lonmax: 0.0 $
	              , latmin: 0.0 $
	              , latmax: 0.0 $
		     }

datastruct = { desc : 0 $
                   , hour : 0.0 $
                   , fov : 0 $
                   , lon : 0.0 $
                   , lat: 0.0 $
                   , landseamask: 0$
                   , tb : 0.0 $
                   }
;
print, 'iii : interogation nb de lignes de ', fullfilename
nrows = file_lines(fullfilename) - 2
;
data = Replicate(datastruct, nrows)
;
; open file
print, 'iii : ouverture pour lecture de ', fullfilename
OPENR, lun, fullfilename, /GET_LUN, ERROR = error
IF (error NE 0) then begin
   ras = report(['eee : can not open for reading'$
         + '!C' $
         + fullfilename])
   return, result
ENDIF
;
; read header lines
READF, lun, header1, format=format_header1
print, header1
READF, lun, header2, format=format_header2
;
print, header2
; read data lines
READF, lun, data, format=format_data
;
; close file
FREE_LUN, lun
;
result={header1: header1, header2: header2, data: data}
;
return, result
;
END
