; docformat = 'rst'
;+
;
; .. _forout.pro:
;
; ==========
; forout.pro
; ==========
;
; returns a file name writable in :file:`${PROJECT_OD}`
;
; :Categories: outputs
;
; :Uses:
;  :func:`report <saxo:report>`
;
; :Params:
;  name : in, required, type=string
;   basename of the file to create in :file:`${PROJECT_OD}/`
;
; :Keywords:
;  OVERWRITE : default=false
;   to overwrite the output image file if exists.
;
; :Returns:
;  filename (dirname and basename) or -1 if error
;
; :Restrictions:
;  - Requires SAXO
;
; :Pre:
;  :ref:`varamma_profile.sh`
;
; :Post:
;  :ref:`varamma_profile.sh`
;
; :Todo:
;
;
; :Examples:
;
; To prepare the production of :file:`${PROJECT_OD}/figure08.png`::
;
;    fullfilename_out=forout('figure08.png',/OVERWRITE)
;    print, fullfilename_out
;   /usr/temp/fplod/varamma_d/figure08.png
;
; If asked an other time without /OVERWRITE, the file is not anymore writable::
;
;    fullfilename_out=forout('figure08.png')
;    print, fullfilename_out
;   -1
;
; :History:
;
;  $Id: forout.pro 215 2011-04-26 10:33:10Z pinsard $
;
; - fplod 2008-08-12T15:26:26Z aedon.locean-ipsl.upmc.fr (Darwin)
;
;   * creation from hovmuller_latitude.pro
;
;-
FUNCTION forout, name, OVERWRITE=overwrite, _EXTRA=extra
;
compile_opt idl2, strictarrsubs
;
@cm_project
;
nparam = N_PARAMS()
IF (nparam LT 1) THEN BEGIN
 ras = report('Incorrect number of arguments.')
 RETURN,-1
ENDIF
;
; test if name is a string
IF size(name, /type) NE 7 THEN BEGIN
 ras = report('name must be a string')
 RETURN, -1
ENDIF
;
filename_out=name
;
IF (keyword_set(overwrite)) THEN BEGIN
 overwrite = 1
ENDIF ELSE BEGIN
 overwrite=0
ENDELSE
;
; test if ${PROJECT_OD} defined
CASE project_od_env OF
 '' : BEGIN
  msg = 'eee : ${PROJECT_OD} is not defined'
  ras = report(msg)
  RETURN, -1
      END
 ELSE: BEGIN
  msg = 'iii : ${PROJECT_OD} is ' + project_od_env
  ras = report(msg)
       END
ENDCASE

; check if output data will be possible
iodirout = isadirectory(project_od_env)
;
; existence and protection
IF (FILE_TEST(iodirout, /DIRECTORY, /WRITE) EQ 0) THEN BEGIN
 msg = 'eee : the directory' + iodirout  + ' was not found.'
 ras = report(msg)
 RETURN, -1
ENDIF
;
fullfilename_out = iodirout + filename_out
;
; in order to avoid unexpected overwritten
IF (overwrite EQ 0) THEN BEGIN
 IF (FILE_TEST(fullfilename_out) EQ 1) THEN BEGIN
  msg = 'eee : the file ' + fullfilename_out  + ' already exists.'
  ras = report(msg)
  RETURN, -1
 ENDIF
ENDIF
;
RETURN, fullfilename_out
;
END
