function [result] = histo_cumulEPSAT(PluieEPSAT, Annee, Mois, Jour, nbclasses)

%HISTO_CUMULEPSAT trace la distribution des prcipitations EPSAT cumules sur
%plusiers pas de temps dans un domaine d'tude donn

%
%+
%
% ==================
% histo_cumulEPSAT.m
% ==================
%
% .. function:: histo_cumulEPSAT(CumulEPSAT, Annee, Mois, Jour, nbclasses)
%
% DESCRIPTION
% ===========
%
% - Donnes d'entres :
%
%   * Annee (en caractres) : Anne (AAAA) du jour
%     associe aux donnes CumulEPSAT
%   * Mois (en caractres) : Numro du mois (MM) du jour
%     associ aux donnes CumulEPSAT
%   * Jour (en caractres) : Numro du jour (JJ) dans le mois
%     associ aux donnes CumulEPSAT
%   * CumulEPSAT : cumul de prcipitations EPSAT estimes sur plusieurs
%     pas de temps dans un doamine d'tude donn
%     dimension 1 : lat,  dimension 2 : lon
%   * nbclasses : Nombres de classes dans l'histogramme.
%
% Trac la distribution des prcipitations EPSAT cumules sur
% plusiers pas de temps dans un domaine d'tude donn
%
% Figure is saved  in :file:`./histo_cumulEPSAT.ps`
%
% EXAMPLES
% ========
%
% To plot histogram of cumul of precipitation from simulated EPSAT data:
%
% see demo 1
%
% To plot histogram of cumul of precipitation from a real EPSAT file
%
% see demo 2
%
% SEE ALSO
% ========
%
% :ref:`data_epsat`
%
% :func:`EPSATread`
% :func:`simul_epsat`
%
% :func:`cumulEPSAT_day`
%
% TODO
% ====
%
% domaine d'etude pas dans les arguments d'entre
%
% explication sur "on en prend 20"
%
% on the plot add yyyymmddhhmn begin and end
% on the plot add lat-lon box
%
% EVOLUTIONS
% ==========
%
% $Id$
%
% $URL: svn+ssh://pinsard@forge.ipsl.jussieu.fr/ipsl/forge/projets/varamma/svn/trunk/src/histo_cumulEPSAT.m $
%
% - fplod 20110830T074359Z cratos (Linux)
%
%   * suppress interactivity
%
% - fplod 20110805T075522Z cratos (Linux)
%
%   * new arguments EPSATread and simul_epsat
%
% - fplod 20110520T101102Z aedon.locean-ipsl.upmc.fr (Darwin)
%
%   * remove diacriticals from title and labels
%     to avoid "warning: ft_render: skipping missing glyph for character"
%     runnning with octave 3.4
%
% - fplod 20110311T110836Z aedon.locean-ipsl.upmc.fr (Darwin)
%
%   * revision of description
%   * add demo with func:`simul_epsat`
%   * suppress fullscreen directive (known problem using octave)
%
% - fplod 20110309T161252Z aedon.locean-ipsl.upmc.fr (Darwin)
%
%   * save figure
%   * correction for octave to see labels and have a visible figure :
%     suppress fontsize and fontname directives
%
% - fplod 20110304T085311Z aedon.locean-ipsl.upmc.fr (Darwin)
%
%   * fix rst header
%
% - jaclod 20110303
%
%   * Ajout de la doc et de la dmonstration
%
% - jaclod 20110226
%
%   * Cration
%
%-
result=-1;
%
usage='[result] = histo_cumulEPSAT(PluieEPSAT, Annee, Mois, Jour, nbclasses)';
if nargin~=nargin(mfilename)
   disp(['Incorrect number of input arguments ' num2str(nargin)]);
   error(usage)
end
%
if nargout ~= nargout(mfilename)
   disp(['Incorrect number of output arguments ' num2str(nargout)]);
   error(usage);
end
%

%%%%%%%%%%%%%
% Paramtres
%%%%%%%%%%%%%

% On place les valeurs sur un seul vecteur.
DistribPrecip = reshape(PluieEPSAT,1,[]);
donnees= find(DistribPrecip>0);

% On dfinit les diffrentes valeurs centrales de l'histogramme. On en
% prend 20.
X = min(DistribPrecip):(max(DistribPrecip)-min(DistribPrecip))/(nbclasses-1):max(DistribPrecip);

% prepare figure
ifigure=gcf();
figure(ifigure);

% On trace l'histogramme.
hist(DistribPrecip(donnees),X);

% Ajout de titres.
xlabel('Precipitations cumulees (mm)')
ylabel('Nombre de points')
title(['Distribution des precipitations EPSAT cumulees pour le ' ([Annee '-' Mois '-' Jour])])
%
% save figure
printer='ps';
print_printer=['-d', printer];
fullfilename=['./', mfilename '.' printer];
clear printer;
print(print_printer,fullfilename);
clear print_printer;
clear fullfilename;

result=0;

end

%!demo
%! % To plot histogram of cumul of precipitation from simulated EPSAT data:
%!
%! clear all
%! close all
%! varamma_startup
%! more off
%! index_simulation = uint8(2);
%! [lon_value, lat_value, datejul_value, datestr_value, epsat_value] = simul_epsat(index_simulation);
%! % First datestr_value is used here as the only date of cumulEPSAT_day
%! Annee = datestr_value(1,1:4);
%! Mois = datestr_value(1,5:6);
%! Jour = datestr_value(1,7:8);
%! [CumulEPSAT] = cumulEPSAT_day(epsat_value);
%! nbclasses = 15;
%! result = histo_cumulEPSAT(CumulEPSAT, Annee, Mois, Jour, nbclasses);

%!demo
% To plot histogram of cumul of precipitation from EPSAT file
% :file:`${PROJECT_ID}/EPSAT/2006/08/surf-rr_epsat-sg_multi-sat_010d_30min_200600801_v3.1-02c.nc`:
%! clear all
%! close all
%! varamma_startup;
%! more off
%! Annee = '2006';
%! Mois = '08';
%! Jour = '01';
%! ncfilesset = 'extracted';
%! [fullfilename] = EPSATbuildfullfilename(ncfilesset, Annee, Mois, Jour);
%! [PluieEPSAT, Temps, Longitudes, Latitudes] = EPSATread(ncfilesset, fullfilename);
%! [CumulEPSAT] = cumulEPSAT_day(PluieEPSAT);
%! nbclasses = 15;
%! result = histo_cumulEPSAT(CumulEPSAT, Annee, Mois, Jour, nbclasses);
