function [result]=plot_precip(station, lon_value, lat_value, datestr_value, precip_value)

%PLOT_PRECIP plot precipitation from precipitation 2D array

%
%+
%
% =============
% plot_precip.m
% =============
%
% .. function:: plot_precip(station, lon_value, lat_value, datestr_value, precip_value)
%
% DESCRIPTION
% ===========
%
% plot precipitions 1 month/page
%
% Figures are saved in :file:`./{station}_{yyyy}{mm}_plot_precip.ps`
%
%    :param station: station
%    :type station: string
%    :raise station: required
%
%    :param lon_value: longitude
%    :type lon_value: float
%    :raise lon_value: required
%
%    :param lat_value: latitude
%    :type lat_value: float
%    :raise lat_value: required
%
%    :param datestr_value: date (yyyymmdd) array
%    :type datestr_value: 1D array of strings
%    :raise datestr_value: required
%
%    :param precip_value: precipitation array
%    :type precip_value: 2D array of floats
%    :raise precip_value: required
%
%    :returns: 0 if ok or -1 if error
%    :rtype: integer
%
% EXAMPLES
% ========
%
% see demos
%
% SEE ALSO
% ========
%
% :ref:`data_precipitation_insitu`
%
% :ref:`data_station_pluviometer`
%
% :func:`read_precip`
%
% :func:`read_station`, :func:`get_station_geolocation`,
% :func:`geolocation_to_string`
%
% TODO
% ====
%
% check arguments (type, size, value)
%
% clean
%
% EVOLUTIONS
% ==========
%
% $Id$
%
% $URL: svn+ssh://pinsard@forge.ipsl.jussieu.fr/ipsl/forge/projets/varamma/svn/trunk/src/plot_precip.m $
%
% - fplod 20110816T073113Z aedon.locean-ipsl.upmc.fr (Darwin)
%
%   * replace int16 by uint16 for integer parameters
%   * replace int8 by uint8 for integer parameters
%
% - fplod 20110302T084330Z aedon.locean-ipsl.upmc.fr (Darwin)
%
%   * test inconsistent dimensions
%
% - fplod 20110224T091800Z aedon.locean-ipsl.upmc.fr (Darwin)
%
%   * move examples to demo
%   * fix demo bug
%
% - fplod 20110223T104250Z aedon.locean-ipsl.upmc.fr (Darwin)
%
%   * usage of geolocation_to_string
%
% - fplod 20110221T164420Z aedon.locean-ipsl.upmc.fr (Darwin)
%
%   * add usage of :func:`read_station` ou :func:`simul_station` and
%     :func:`get_station_geolocation` in examples
%
% - fplod 20110124T144154Z aedon.locean-ipsl.upmc.fr (Darwin)
%
%   * better missing value handling
%   * constraint on y axis min and max
%
% - fplod 20110117T143947Z aedon.locean-ipsl.upmc.fr (Darwin)
%
%   * creation inspired by exercise precipPlot.m
%
%-
%
result=-1;
%
usage='result=plot_precip(station, lon_value, lat_value, datestr_value, precip_value)';
%
if nargin~=nargin(mfilename)
   disp(['Incorrect number of input arguments ' num2str(nargin)]);
   error(usage);
end
%
if nargout ~= nargout(mfilename)
   disp(['Incorrect number of output arguments ' num2str(nargout)]);
   error(usage);
end
%
arg_info=whos('station');
if ~strcmp(arg_info.class,'char')
   disp(['Incorrect type of arg station ' arg_info.class]);
   whos station
   error(usage);
end
clear arg_info
%
arg_info=whos('lon_value');
if ~strcmp(arg_info.class,'double')
   disp(['Incorrect type of arg lon_value ' arg_info.class]);
   whos lon_value
   error(usage);
end
%
if ~isvector(lon_value)
   disp('arg lon_value must be a vector');
   error(usage);
end
%
arg_info=whos('lat_value');
if ~strcmp(arg_info.class,'double')
   disp(['Incorrect type of arg lat_value ' arg_info.class]);
   whos lat_value
   error(usage);
end
clear arg_info
%
if ~isvector(lat_value)
   disp('arg lat_value must be a vector');
   error(usage);
end
%
if (size(datestr_value,1) ~= size(precip_value,2))
   disp(['inconsistent dimensions of datestr_value and precip_value']);
   whos datestr_value
   whos precip_value
   error(usage);
end
%
% ++ check arguments (type, size, values)
%
clear usage
%
% the maximum of geolocation is 1 here
index_geolocation=1;
%
nb_datestr=size(datestr_value,1);
% calculate minimal/max month and year
datenum_value=zeros(nb_datestr,1,'double');
datenum_value(:)=NaN;
for index_datestr=1:nb_datestr
   datenum_value(index_datestr)=datenum(datestr_value(index_datestr,:),'yyyymmdd');
end
clear index_datestr
datenum_min=min(datenum_value);
mm_min=str2double(datestr(datenum_min,'mm'));
yyyy_min=datestr(datenum_min,'yyyy');
clear datenum_min
datenum_max=max(datenum_value);
clear datenum_value
mm_max=str2double(datestr(datenum_max,'mm'));
yyyy_max=datestr(datenum_max,'yyyy');
clear datenum_max
%
% check only one year
if ~strcmp(yyyy_min,yyyy_max)
   disp([' yyyy_min = ' yyyy_min ' yyyy_max = ' yyyy_max]);
   error('eee : yyyy_min and yyyy_max different')
end
clear yyyy_max
dyyyy=str2double(yyyy_min);
clear yyyy_min
%
% the maximum of day is the number of days in a month
days_in_month_max = 31;
% the maximum of month is the number of months in a year
months_in_year_max = 12;
% preallocation of precip_rd_cm array to missing value NaN
% note : precip_rd_cm for precip array arrange with column = month and row = day
precip_rd_cm=repmat(NaN,days_in_month_max,months_in_year_max);
%
clear days_in_month_max
clear months_in_year_max
%
% loop on date
for index_date=1:nb_datestr
    dd_value = str2double(datestr_value(index_date,7:8));
    mm_value = str2double(datestr_value(index_date,5:6));
    precip_neg=(precip_value(index_geolocation,index_date) < 0.);
    precip_nan=isnan(precip_value(index_geolocation,index_date));
    precip_mv=or(precip_nan,precip_neg);
    if (~precip_mv)
       precip_rd_cm(dd_value,mm_value)=precip_value(index_geolocation,index_date);
       % debug disp(['pas mv ' num2str(dd_value) ' ' num2str(mm_value) ' ' num2str(precip_rd_cm(dd_value,mm_value))] );
    end
    clear mm_value
    clear dd_value
    clear precip_neg
    clear precip_nan
    clear precip_mv
end
clear index_date
clear index_geolocation
clear nb_datestr
%
ymin=0.0;
ymax=max(max(precip_rd_cm));
ifigure=1;
for imonth=mm_min:mm_max
   dd_max=eomday(dyyyy,imonth);
   precip_rd_cm(1:dd_max,imonth);
   ydata=precip_rd_cm(1:dd_max,imonth);
   % ++ test if ydata is not only fill with NaN
   xdata=datenum(dyyyy,imonth,1):datenum(dyyyy,imonth,dd_max);
   clear dd_max
   figure(ifigure);
   plot(xdata,ydata,'o')
   clear ydata
   set(gca,'XTick',xdata)
   % nb : because of lack of datetick in octave 3.0.2 on zeus
   % we use xticklabel instead of datetick
   xticklabel=datestr(xdata,'dd');
   set(gca,'XTickLabel',xticklabel)
   clear xticklabel
   % define labels and title
   xlabel(['days in month ' num2str(imonth,'%2.2d') ' of the year ' num2str(dyyyy,'%4.4d')]);
   ylabel('mean precipitation (mm)');
   geolocation_station=geolocation_to_string(lon_value(1),lat_value(1));
   title(['daily precipitation at ' station ' ' geolocation_station]);
   clear geolocation_station
   xlim([xdata(1) xdata(size(xdata,2))]);
   clear xdata
   ylim([ymin ymax]);
   %
   % save figure
   printer='ps';
   print_printer=['-d', printer];
   fullfilename=['./', station '_' num2str(dyyyy,'%4.4d') num2str(imonth,'%2.2d') '_' mfilename '.' printer];
   clear printer;
   print(print_printer,fullfilename);
   clear print_printer;
   clear fullfilename;
   %
   ifigure=ifigure + 1;
end
clear precip_rd_cm
clear imonth
clear dyyyy
clear ifigure
clear mm_min
clear mm_max
%
%++ whos
%
result=0;
%
end

%!demo
%! % From scratch to plot some simulated precipitation values of simulated stations::
%
%! close all
%! varamma_startup
%! more off
%! station = 'station01';
%! index_simulation = uint8(1);
%! [datestr_value, precip_value] = simul_precip(station, index_simulation);
%! [lon_value, lat_value, altitude_value, code_value, station_value] = simul_station(index_simulation);
%! [lon_station, lat_station, altitude_station] = get_station_geolocation(station, lon_value, lat_value, altitude_value, station_value);
%! result = plot_precip(station, lon_station, lat_station, datestr_value, precip_value)
%
%!demo
%! % To plot precipitation from :file:`${PROJECT_ID}/PRECIP/barkedji_pluie_2006.csv`::
%
%! close all
%! varamma_startup
%! more off
%! yyyy = uint16(2006);
%! station = 'barkedji';
%! datasettype = 'precip_cm'
%! [datestr_value, precip_value] = read_precip(station,yyyy,datasettype);
%! [lon_value, lat_value, altitude_value, code_value, station_value, region_value] = read_station();
%! [lon_station, lat_station, altitude_station] = get_station_geolocation(station, lon_value, lat_value, altitude_value, station_value);
%! result = plot_precip(station, lon_station, lat_station, datestr_value, precip_value)
