function [result]=plot_station(lonmin, lonmax, latmin, latmax, lon_value, lat_value, code_value, station_value, region_value)

%PLOT_STATION plot stations on a world map

%+
%
% ==============
% plot_station.m
% ==============
%
% .. function:: plot_station(lonmin, lonmax, latmin, latmax, lon_value, lat_value, code_value, station_value, region_value)
%
% DESCRIPTION
% ===========
%
% Plot stations on a world map delimited by **lonmin**, **lonmax**, **latmin**,
% **latmax**.
%
% Figure is saved in :file:`./plot_precip.ps`
%
%   :param lonmin: longitude min of the plot, W < 0
%   :units lonmin: deg
%   :type lonmin: double
%   :raise lonmin: required
%
%   :param lonmax: longitude max of the plot, W < 0
%   :units lonmax: deg
%   :type lonmax: double
%   :raise lonmax: required
%
%   :param latmin: latitude min of the plot, N > 0
%   :units latmin: deg
%   :type latmin: double
%   :raise latmin: required
%
%   :param latmax: latitude max of the plot, N > 0
%   :units latmax: deg
%   :type latmax: double
%   :raise latmax: required
%
%   :param lon_value: longitude array
%   :type lon_value: 1D array of floats
%   :raise lon_value: required
%
%   :param lat_value: latitude array
%   :type lat_value: 1D array of floats
%   :raise lat_value: required
%
%   :param code_value: code array
%   :type code_value: 1D array of floats
%   :raise code_value: required
%
%   :param station_value: station name array
%   :type station_value: 1D array of strings
%   :raise station_value: required
%
%   :param region_value: region name array
%   :type region_value: 1D array of strings
%   :raise region_value: required
%
%   :returns: 0 if ok or -1 if error
%   :rtype: integer
%
% EXAMPLES
% ========
%
% From scratch to plot some simulated pluviometer stations:
%
% see demo 1
%
% To plot precipitation from :file:`${PROJECT_ID}/PRECIP/stations.csv`:
%
% see demo 2
%
% SEE ALSO
% ========
%
% :ref:`guide data station pluviometer <data_station_pluviometer>`
%
% :ref:`varamma_startup.m`
%
% :func:`read_station`
% :func:`simul_station`
%
% TODO
% ====
%
% check arg
%
% altitude missing value
%
% ref to traitdecode
%
% * add region boxes : pb = annotation n'existe pas sous octave et
%   les units de position de annotation matlab ne sont pas
%   pratiques (entre 0. et 1.)
%
% EVOLUTIONS
% ==========
%
% $Id$
%
% $URL: svn+ssh://pinsard@forge.ipsl.jussieu.fr/ipsl/forge/projets/varamma/svn/trunk/src/plot_station.m $
%
% - fplod 20110302T100637Z aedon.locean-ipsl.upmc.fr (Darwin)
%
%   * adapation for matlab : can not use octave size_equal
%
% - fplod 20110302T084421Z aedon.locean-ipsl.upmc.fr (Darwin)
%
%   * add usage
%   * test inconsistent dimensions of arguments
%
% - fplod 20110224T083732Z aedon.locean-ipsl.upmc.fr (Darwin)
%
%   * add missing end statement
%   * add demos (copy of examples)
%
% - fplod 20110218T101523Z aedon.locean-ipsl.upmc.fr (Darwin)
%
%   * force axis +2 deg each side
%   * add a polygon to see area
%   * add warnings if some station is outside of map box
%   * add warnings if some station geolocation (lat or lon) has NaN
%
% - fplod 20110216T091520Z aedon.locean-ipsl.upmc.fr (Darwin)
%
%   * add code (not used yet)
%   * fix result value if ok
%
% - fplod 20110215T161745Z aedon.locean-ipsl.upmc.fr (Darwin)
%
%   * creation
%
%-
global application;
%
result=-1;
%
usage='[result]=plot_station(lonmin, lonmax, latmin, latmax, lon_value, lat_value, code_value, station_value, region_value)';
%
if nargin~=nargin(mfilename)
   disp(['Incorrect number of input arguments ' num2str(nargin)]);
   error(usage);
end
%
if nargout ~= nargout(mfilename)
   disp(['Incorrect number of output arguments ' num2str(nargout)]);
   error(usage);
end
%
arg_info=whos('lonmin');
if ~strcmp(arg_info.class,'double')
   disp(['Incorrect type of arg lonmin ' arg_info.class]);
   whos lonmin
   error(usage);
end
%
arg_info=whos('lonmax');
if ~strcmp(arg_info.class,'double')
   disp(['Incorrect type of arg lonmax ' arg_info.class]);
   whos lonmax
   error(usage);
end
%
arg_info=whos('latmin');
if ~strcmp(arg_info.class,'double')
   disp(['Incorrect type of arg latmin ' arg_info.class]);
   whos latmin
   error(usage);
end
%
arg_info=whos('latmax');
if ~strcmp(arg_info.class,'double')
   disp(['Incorrect type of arg latmax ' arg_info.class]);
   whos lonmax
   error(usage);
end
%
arg_info=whos('lon_value');
if ~strcmp(arg_info.class,'double')
   disp(['Incorrect type of arg lon_value ' arg_info.class]);
   whos lon_value
   error(usage);
end
clear arg_info
%
if ~isvector(lon_value)
   disp('arg lon_value must be a vector');
   error(usage);
end
%
arg_info=whos('lat_value');
if ~strcmp(arg_info.class,'double')
   disp(['Incorrect type of arg lat_value ' arg_info.class]);
   whos lat_value
   error(usage);
end
clear arg_info
%
if ~isvector(lat_value)
   disp('arg lat_value must be a vector');
   error(usage);
end
%
arg_info=whos('code_value');
if ~strcmp(arg_info.class,'double')
   disp(['Incorrect type of arg code_value ' arg_info.class]);
   whos code_value
   error(usage);
end
clear arg_info
%
if ~isvector(code_value)
   disp('arg code_value must be a vector');
   error(usage);
end
%
arg_info=whos('station_value');
if ~strcmp(arg_info.class,'cell')
   disp(['Incorrect type of arg station_value ' arg_info.class]);
   whos station_value
   error(usage);
end
%
clear arg_info
arg_info=whos('region_value');
if ~strcmp(arg_info.class,'cell')
   disp(['Incorrect type of arg region_value ' arg_info.class]);
   whos region_value
   error(usage);
end
clear arg_info
%

switch application
  case {'matlab'}
     test_consistent=isequal(numel(lon_value), numel(lat_value), numel(code_value), numel(station_value), numel(region_value));
  case {'octave'}
     test_consistent=size_equal(lon_value, lat_value, code_value, station_value, region_value);
  otherwise
     error('unknown application running');
end
%
if (not(test_consistent))
   disp(['eee : inconsistent dimensions of lon_value, lat_value, code_value, station_value and region_value']);
   whos lon_value
   whos lat_value
   whos code_value
   whos station_value
   whos region_value
   error(usage);
end
%
% define axis not to close from area to be plot
ymin=latmin - 2;
ymax=latmax + 2;
xmin=lonmin - 2;
xmax=lonmax + 2;
%
% read world map
load('cmgco_traitdecote.mat','Xgco','Ygco');
%
% initialiaze a new figure
ifigure=gcf()+1;
figure(ifigure);
%
% plot world map (black dots)
plot(Xgco,Ygco,'k.','MarkerSize',1)
clear Xgco;
clear Ygco
% limit world map around latmin,latmax and lonmin, lonmax
axis([lonmin lonmax latmin latmax]);
%
% define an area (polygon) to check if station is inside it
x_area_region=[lonmin lonmax lonmax lonmin lonmin];
y_area_region=[latmin latmin latmax latmax latmin];
% for each station, write name on the map at lat and lon position
for index_station=1:size(station_value,1)
  label=station_value(index_station,:);
  % check if geolocation of the station has NaN
  if (isnan(lon_value(index_station)) |  isnan(lat_value(index_station)))
     warning([ 'station #' num2str(index_station) ' NaN geolocation']);
     disp([ 'lon_value : ' num2str(lon_value(index_station))]);
     disp([ 'lat_value : ' num2str(lat_value(index_station))]);
  else
     % check if station is inside area
     in_area = inpolygon(lon_value(index_station),lat_value(index_station),x_area_region,y_area_region);
     if (~in_area)
        warning([ 'station #' num2str(index_station) ' outside of the box']);
        disp([ 'lon_value : ' num2str(lon_value(index_station))]);
        disp([ 'lat_value : ' num2str(lat_value(index_station))]);
        disp([ 'box: ' '[', num2str(lonmin) ',' num2str(latmin) ']' '[' num2str(lonmax) ',' num2str(latmax) ']' ]);
     else
        text(lon_value(index_station),lat_value(index_station),label);
     end
     clear in_area
     clear label
  end
end
%
hold on
plot(x_area_region,y_area_region,'-')
%
% limit world map around latmin,latmax and lonmin, lonmax + 2 on each side
axis([xmin xmax ymin ymax]);
clear xmin;
clear xmax;
clear ymin;
clear ymax;
%
clear lonmin;
clear lonmax;
clear latmin;
clear latmax;
clear x_area_region;
clear y_area_region;
%
% save figure
printer='ps';
print_printer=['-d', printer];
fullfilename=['./', mfilename, '.', printer];
clear printer;
print(print_printer,fullfilename);
clear print_printer;
clear fullfilename;
%
result=0;
%
end

%!demo
%!
%! % From scratch to plot some simulated pluviometer stations::
%!
%! clear all
%! close all
%! varamma_startup
%! lonmin = -20;
%! lonmax = -10;
%! latmin = 10;
%! latmax = 20;
%! index_simulation = uint8(1);
%! [lon_value, lat_value, altitude_value, code_value, station_value, region_value] = simul_station(index_simulation);
%! result = plot_station(lonmin, lonmax, latmin, latmax, lon_value, lat_value, code_value, station_value, region_value);

%!demo
%!
%! % To plot precipitation from :file:`${PROJECT_ID}/PRECIP/stations.csv`::
%!
%! clear all
%! close all
%! varamma_startup
%! lonmin = -20;
%! lonmax = -10;
%! latmin = 10;
%! latmax = 20;
%! [lon_value, lat_value, altitude_value, code_value, station_value, region_value] = read_station();
%! result = plot_station(lonmin, lonmax, latmin, latmax, lon_value, lat_value, code_value, station_value, region_value);
