function [lon_value, lat_value, altitude_value, code_value, station_value, region_value]=read_station()

%READ_STATION fill station associated arrays date, lat, lon
%with data in file :file:`${PROJECT_ID}/PRECIP/stations.csv`

%
%+
%
% ==============
% read_station.m
% ==============
%
% .. function:: read_station()
%
% DESCRIPTION
% ===========
%
% read station pulviometer data in a
% file :file:`${PROJECT_ID}/PRECIP/stations.csv`
%
% Les longitudes sont stockes dans le tableau 1D **lon_value**.
%
% Les latitudes sont stockes dans le tableau 1D **lat_value**.
%
% Les altitudes sont stockes dans le tableau **altitude_value**.
%
% Les noms sont stockes dans le tableau **station_value**.
%
% Les regions sont stockes dans le tableau **region_value**.
%
% EXAMPLES
% ========
%
% To read :file:`${PROJECT_ID}/PRECIP/stations.csv`:
%
% see demo 1
%
% A file :file:`${PROJECT_ID}/PRECIP/stations_write_station.csv` can be written
% with data we just read::
%
%  octave> result=write_station(station,lon_value, lat_value, code_value, station_value, region_value);
%
% Files comparison might help detecting problems (either on files or in programs involved)::
%
%  $ diff ${PROJECT_ID}/PRECIP/stations.csv ${PROJECT_ID}/PRECIP/stations_write_station.csv
%
% SEE ALSO
% ========
%
% :ref:`data_station_pluviometer`
%
% :ref:`data_precipitation_insitu`
%
%
% :func:`simul_station`
% :func:`write_station`
% :func:`plot_station`
%
% :func:`plot_precip`
%
% TODO
% ====
%
% check header
%
% check lat or lon missing value
%
% status de retour
%
% description du rsultat en ReST
%
% add test : no yet done because too many assert to write
%
% EVOLUTIONS
% ==========
%
% $Id$
%
% $URL: svn+ssh://pinsard@forge.ipsl.jussieu.fr/ipsl/forge/projets/varamma/svn/trunk/src/read_station.m $
%
% - fplod 20110224T102211Z aedon.locean-ipsl.upmc.fr (Darwin)
%
%   * remove code from examples (already in demo)
%
% - fplod 20110223T151505Z aedon.locean-ipsl.upmc.fr (Darwin)
%
%   * add demo
%
% - fplod 20110218T101238Z aedon.locean-ipsl.upmc.fr (Darwin)
%
%   * init station_value and region_value to NaN
%
% - fplod 20110217T141928Z aedon.locean-ipsl.upmc.fr (Darwin)
%
%   * find solution to find missing station_value
%   * find solution to reduce preallocated arrays
%
% - fplod 20110217T120019Z aedon.locean-ipsl.upmc.fr (Darwin)
%
%   * usage of cells array of string to handle station_value and code_value
%     strings array are real nightmare (dealing with indices)
%
% - fplod 20110215T165039Z aedon.locean-ipsl.upmc.fr (Darwin)
%
%   * creation
%
%-
%
global PROJECT_ID
%
usage='[lon_value, lat_value, altitude_value, code_value, station_value, region_value]=read_station()';
%
if nargin~=nargin(mfilename)
   disp(['Incorrect number of input arguments ' num2str(nargin)]);
   error(usage);
end
%
if nargout ~= nargout(mfilename)
   disp(['Incorrect number of output arguments ' num2str(nargout)]);
   error(usage);
end
%
% build station filename to be read
%++fullfilename=[PROJECT_ID 'PRECIP/' 'stations_write_station.csv'];
fullfilename=[PROJECT_ID 'PRECIP/' 'stations.csv'];
disp(['iii : opening for reading ', fullfilename]);
% opening file
[fid,message]=fopen(fullfilename,'r');
% test if fullfilename exists
if (fid == -1)
    disp(message)
    error([ fullfilename ' not exist'])
end
clear fullfilename
clear message
% the maximum of station is set to 400
nb_station_max = 400;
%
% preallocation of station_value array to missing value
station_value=repmat({'NaN'},nb_station_max,1);
% preallocation of region_value array to missing value
region_value=repmat({'NaN'},nb_station_max,1);
%
% preallocation of code_value array to missing value NaN
code_value=zeros(nb_station_max,1,'double');
code_value(:)=NaN;
% preallocation of altitude_value array to missing value NaN
altitude_value=zeros(nb_station_max,1,'double');
altitude_value(:)=NaN;
% preallocation of lat_value array to missing value NaN
lat_value=zeros(nb_station_max,1,'double');
lat_value(:)=NaN;
% preallocation of lon_value array to missing value NaN
lon_value=zeros(nb_station_max,1,'double');
lon_value(:)=NaN;
%
% read file
%
% first line contains "Code,Station-Nom,Longitude,Latitude,Altitude,Region"
%
% get next line as a string
header_line = fgetl(fid);
% +todo+ check header
clear header_line
%
% reading data from file (from second line till end of file)
index_line_data=1;
%
while (feof(fid)~= 1)
  data_line = fgetl(fid);
  % loop to handle eahc item on the data line (comma separated value)
  index_item=1;
  while ( ~isempty(data_line) )
     % parse next column label
     [item,data_line] = strtok(data_line,',');
     if (index_item == 1 )
       code_value(index_line_data)=str2double(item);
     elseif (index_item == 2)
       station_value(index_line_data)=cellstr(item);
     elseif (index_item == 3)
       lon_value(index_line_data)=str2double(item);
     elseif (index_item == 4)
       lat_value(index_line_data)=str2double(item);
     elseif (index_item == 5)
       altitude_value(index_line_data)=str2double(item);
     elseif (index_item == 6)
       region_value(index_line_data)=cellstr(item);
     else
       error([ 'invalid data line (too many items) : ' data_line item]);
     end
     clear item
     % increase index of item
     index_item = index_item + 1;
  end
  % check for number of item in current data line
  if (index_item < 6)
     error([ 'invalid data line (missing items) :' data_line])
  end
  index_line_data=index_line_data+1;
  clear data_line
  clear index_item
end
if (index_line_data > nb_station_max+1)
  error([' need to increase nb_station_max up to ' num2str(index_line_data) ]);
end
%
clear index_line_data
%
fclose(fid);
clear fid
%
% remove extra data NaN values due to preallocation
% criteria = when station_value is not 'NaN'
missing_station_indices=find(ismember(station_value, 'NaN')==1);
nb_station=min(missing_station_indices)-1;
code_value=code_value(1:nb_station);
station_value=station_value(1:nb_station);
lat_value=lat_value(1:nb_station);
lon_value=lon_value(1:nb_station);
altitude_value=altitude_value(1:nb_station);
region_value=region_value(1:nb_station);
%
clear nb_station_max
clear nb_station
%
%++whos
%
result=0;
end

%!demo
%! varamma_startup
%! more off
%! [lon_value, lat_value, altitude_value, code_value, station_value, region_value] = read_station()
