#!/usr/bin/env python
# -*- coding: iso-8859-1 -*-

__docformat__ = 'reStructuredText'
__autodoc__ = ['read_zi']

import os
import sys
import csv

def read_zi():

    """

    =========
    read_zi.m
    =========

    .. :py:function:: read_zi()

    DESCRIPTION
    ===========

    read zone of interest data in a
    file :file:`${PROJECT_ID}/maps/varamma_zi.csv`

    Parameters
    ----------

    None

    Returns
    -------

    west_value : array_like
       west most longitudes

    east_value : array_like
       east most longitudes

    north_value : array_like
       north most latitudes

    south_value : array_like
       south most latitudes

    zi_code : array_like of strings
       codes of the zone of interest

    zi_description: array_like of strings
       descriptions of the zone of interest

    Raises
    ------

    EXAMPLES
    ========

    To read :file:`${PROJECT_ID}/maps/varamma_zi.csv`, write a copy in
    :file:`${PROJECT_ID}/maps/varamma_zi_write_zi_py.csv` and compare those
    two files to help detecting problems (either on files or in programs
    involved)::

    >>> (west_value, east_value, north_value, south_value, zi_code, zi_description) = read_zi()
    iii : opening for reading /usr/work/incas/fplod/varamma_d//maps/varamma_zi.csv
    >>> from write_zi import write_zi
    >>> result = write_zi(west_value, east_value, north_value, south_value, zi_code, zi_description)
    iii : opening for writing /usr/work/incas/fplod/varamma_d//maps/varamma_zi_write_zi_py.csv
    >>> import filecmp
    >>> project_id_env = os.environ['PROJECT_ID']
    >>> # build zi filenames to be compared
    >>> original = project_id_env + '/maps/' + 'varamma_zi.csv'
    >>> writed = project_id_env + '/maps/' + 'varamma_zi_write_zi_py.csv'
    >>> filecmp.cmp(original,writed)
    True

    SEE ALSO
    ========

    :ref:`zones`

    :func:`simul_zi`
    :func:`write_zi`
    :func:`plot_zi`

    TODO
    ====

    test failed because of newline \r\n writed by wrtite_zi.py

    get rid of matlab stuff

    include in sphinx processing

    EVOLUTIONS
    ==========

    $Id$

    $URL$

    - fplod 20110907T121831Z cratos (Linux)

      * try to apply "A Guide to NumPy/SciPy Documentation"
        https://github.com/numpy/numpy/blob/master/doc/HOWTO_DOCUMENT.rst.txt

    - fplod 20110907T100947Z cratos (Linux)

      * fix for test ok (swap west/east, no more preallocation)
      * improve python style (zip) thanks to
        http://www.linuxjournal.com/content/handling-csv-files-python

        this change allow more flexibility in order of rows and
        better checking of header ...

    - fplod 20110906T143708Z cratos (Linux)

      * creation from read_zi.m
      * integration of comparison in test

    """

    usage = '(west_value, east_value, north_value, south_value, zi_code, zi_description)=read_zi()'

    # ++ matlabif nargin~=nargin(mfilename)
    # ++ matlab   disp(['Incorrect number of input arguments ' num2str(nargin)])
    # ++ matlab   error(usage)
    # ++ matlabend

    # ++ matlabif nargout ~= nargout(mfilename)
    # ++ matlab   disp(['Incorrect number of output arguments ' num2str(nargout)])
    # ++ matlab   error(usage)
    # ++ matlabend

    project_id_env = os.environ['PROJECT_ID']

    # build zi filename to be read
    fullfilename = project_id_env + '/maps/' + 'varamma_zi.csv'
    print('iii : opening for reading %s' % (fullfilename))

    # opening file
    try:
        fid = open(fullfilename,"rb")
    except IOError, err:
        sys.stderr.write("Error: %r. %s.\n" % (err.filename, err.strerror))
        sys.exit(1)

    # read file
    reader = csv.reader(fid, quoting=csv.QUOTE_NONNUMERIC)
    headers = reader.next()
    west_value = []
    east_value = []
    north_value = []
    south_value = []
    zi_description = []
    zi_code = []

    for row in reader:
        for header, col in zip(headers, row):
            # ++ debug print "%-8s: %s" % (header, col)
            if header == "code":
                zi_code.append(col)
            elif header == "description":
                zi_description.append(col)
            elif header == "north":
                north_value.append(float(col))
            elif header == "south":
                south_value.append(float(col))
            elif header == "east":
                east_value.append(float(col))
            elif header == "west":
                west_value.append(float(col))
            else:
                raise ValueError("invalid header : '%s'" % header)

    fid.close()

    # first line contains "code,description,north,south,east,west"

    # matlab++  # check for number of item in current data line
    # matlab++  if (index_item < 6)
    # matlab++    error([ 'invalid data line (missing items) :' data_line])
    # matlab++  end
    # matlab++  index_line_data=index_line_data+1
    # matlab++end
    # matlab++if (index_line_data > nb_zi_max+1)
    # matlab++  error([' need to increase nb_zi_max up to ' num2str(index_line_data) ])
    # matlab++end

    return west_value, east_value, north_value, south_value, zi_code, zi_description

# Run main, if called from the command line
if __name__ == '__main__':

    """

    if read_zi.py is launched from the prompt, test is done.

    """

    import doctest
    doctest.testmod()
