#! /bin/sh
#+
#
# .. program:: traite_msg-prod.sh
#
# .. _traite_msg-prod.sh:
#
# ==================
# traite_msg-prod.sh
# ==================
#
# SYNOPSIS
# ========
#
# ::
#
#  $ traite_msg-prod.sh -p product -b yyyymmdd -e yyyymmdd
#
# DESCRIPTION
# ===========
#
# .. option:: -p product name classif, pressure, height, temperature, tb087 ,tb097, tb108, tb108z, tb120, tb134
#
# ``traite_msg-prod.sh`` read :file:`${PROJECT_ID}/MSG/{yyyy}/{mm}/{dd}/{product}_mgs01_{yyyymmdd}_{hhmn}_?????.raw.gz`
# and :file:`${PROJECT_ID}/MSG/{yyyy}/{mm}/{dd}/{product}_mgs01_{yyyymmdd}_{hhmn}_?????.def.gz`.
#
# With these two files ``traite_msg-prod.sh`` produces
# a NetCDF file :file:`${PROJECT_ID}/MSG/{yyyy}/{mm}/{dd}/{yyyymmddmmmn}"_msg-{product}_map_15min.nc` and
# a GIF file :file:`${PROJECT_ID}/MSG/{yyyy}/{mm}/{dd}/{yyyymmddmmmn}"_msg-{product}_map_15min.gif`.
#
# ++ box reduction
#
# .. option:: -f
#
# If :option:`-f` option is set, previously output files will be overwritten.
#
# .. option:: -b beginning date <yyyymmdd>
# .. option:: -e end date <yyyymmdd>
#
# Log file is written on :file:`${PROJECT_LOG}/traite_msg-prod.log.{YYYYMMDDTHHMMSSZ}`
#
# .. only:: man
#
#    Figure is visible on PDF and HTML documents only.
#
# .. only:: html or latex
#
#     .. graphviz::
#
#        digraph traite_msg_prod {
#           graph [
#           rankdir="TB",
#           ]
#
#           msg_t1 [shape=ellipse,fontname=Courier,label="${PROJECT_ID}/MSG/yyyy/mm/dd/prod_msg01_yyyymmdd_hhmn_xxxxx.raw.gz"];
#           msg_t2 [shape=ellipse,fontname=Courier,label="${PROJECT_ID}/MSG/yyyy/mm/dd/prod_msg01_yyyymmdd_hhmn_xxxxx.def.gz"];
#
#           msg_t5 [shape=ellipse,fontname=Courier,label="${PROJECT_ID}/MSG/yyyy/mm/dd/yyyymmddhhmn_msg-prod_map_15min.nc"];
#           msg_t7 [shape=ellipse,fontname=Courier,label="${PROJECT_ID}/MSG/yyyy/mm/dd/yyyymmddhhmn_msg-prod_map_15min.gif"];
#
#           log [shape=ellipse,fontname=Courier,label="${PROJECT_LOG}/traite_msg_prod.log{YYYYMMDDTHHMMSSZ}"];
#
#           traite_msg_prod [shape=box,
#           fontname=Courier,
#           color=blue,
#           URL="http://forge.ipsl.jussieu.fr/varamma/browser/trunk/src/traite_msg-prod.sh",
#           label="${PROJECT}/src/traite_msg-prod.sh"];
#
#           {msg_t1 msg_t2} -> {traite_msg_prod} -> {log msg_t5 msg_t7}
#         }
#
# EXAMPLES
# ========
#
# You already used :ref:`get_msg.sh` to get MSG reference data
# during [20060801, 20060802[ and you want to produce NetCDF files and
# GIF files::
#
#   $ traite_msg-prod.sh -p tb108 -b 20060801 -e 20060802
#
# And look at log file with ::
#
#  $ tlogd.sh traite_msg-prod
#
# and of course on files in ${PROJECT_ID}.
#
# SEE ALSO
# ========
#
# :ref:`guide data MSG <data_msg>`
#
# :ref:`varamma_profile.sh`
#
# Previous step : :ref:`get_msg.sh`
#
# Uses :
# :ref:`create_nc.sh`
# :ref:`create_gif.sh`
#
# Next step : :ref:`concat-files.sh`
#
# .. todo::
#
# build/complete catalog
#
# handle -f option
#
# replace -prod by _prod (graphviz do not like traite_msg-prod and I rather like to have identidy between filename and
# dot entry of the software)
#
# add box in parameter
#
# filename starting with a digit is not recommanded
#
# EVOLUTIONS
# ==========
#
# $Id$
#
# $URL$
#
# - pinsard 20120529T150524Z loholt1.ipsl.polytechnique.fr (Linux)
#
#   * bug fix to handle correctly dates if translation in second on 9 digits
#     (trailing 0 is automaticaly interpreted as an octal number).
#     ex = 20010801
#   * bug fix to handle correctly dates if translation in second of begin
#     and end is not on the same number of digit
#     ex : -b 20010801 -e 20010931
#
# - pinsard 2011-06-01T09:18:41Z loholt1.ipsl.polytechnique.fr (Linux)
#
#   * correction on msg_t3 and msg_t4 terminology (in fact t5 and t7)
#
# - pinsard 2011-05-02T09:59:23Z loholt1.ipsl.polytechnique.fr (Linux)
#
#   * handle errors
#
# - fplod 20110429T144410Z cratos.locean-ipsl.upmc.fr
#
#   * improve time loop like in :ref:`get_msg.sh`
#
# - pinsard 2011-04-28T12:05:09Z loholt1.ipsl.polytechnique.fr (Linux)
#
#   * suppress keepnc
#   * suppress loop on products (product is now a parameter)
#   * get rid of /bdd. ${PROJECT_ID} instead
#   * add yyyymmddb and yyyymmdde parameter
#   * coding rules and consolidation
#
# - fplod 20110427T133133Z cratos.locean-ipsl.upmc.fr (Linux)
#
#   * add {} to design variable
#
# - fplod 20100723T170805Z aedon.locean-ipsl.upmc.fr (Darwin)
#
#   * add template graphic
#
# - fplod 20100710T150000Z
#
#   * change IO using PROJECT_OD
#
# - fplod 20100608T143812Z aedon.locean-ipsl.upmc.fr (Darwin)
#
#   * minimal header
#
# - Karim Ramage ?
#
#-
system=$(uname)
case "${system}" in
   AIX|IRIX64)
     echo "www : no specific posix checking"
     date_cmd=date
   ;;
   Darwin)
     set -o posix
     date_cmd=gdate
   ;;
   Linux)
     set -o posix
     date_cmd=date
   ;;
   *)
    set -o posix
   ;;
esac
unset system
#
set -u
#
LANG=POSIX
#
command=$(basename ${0})
log_date=$(date -u +"%Y%m%dT%H%M%SZ")
#
usage=" Usage : ${command} [-f] -p product -b yyyymmdd -e yyyymmdd"
#
hostname=$(hostname)
#
yyyymmddb_min=20060420
yyyymmdde_max=20061103
#
# default
debug=0
force=0
product=tb108
yyyymmddb=20060420
yyyymmdde=20060420
lonmin=-25
lonmax=25
latmin=-5
latmax=20
#
minargcount=6
if [ ${#} -lt ${minargcount} ]
then
   echo "eee : not enought arguments"
   echo "${usage}"
   exit 1
fi
unset minargcount
#
while [ ${#} -gt 0 ]
do
   case ${1} in
     -p)
        product=${2}
        shift
     ;;
     -b)
        # first date to get
        yyyymmddb=${2}
        shift
     ;;
     -e)
        # last date to get
        yyyymmdde=${2}
        shift
     ;;
     -f)
        force=1
     ;;
     -h)
        echo "${usage}"
        exit 0
     ;;
     *)
        # other choice
        echo "eee : unknown option ${1}"
        echo "${usage}"
        exit 1
     ;;
   esac
   # next flag
   shift
done
unset usage
#
# check parameters
# and define associated variables
${date_cmd} -d "${yyyymmddb}" > /dev/null
status_date=${?}
if [ ${status_date} -ne 0 ]
then
    echo "${command} : eee : yyyymmddb ${yyyymmddb} argument invalid"
    exit 1
fi
unset status_date
#
if [[ "${yyyymmddb}" < "${yyyymmddb_min}" ]]
then
    echo "${command} : eee : yyyymmddb ${yyyymmddb} must be equal or greater than yyyymmddb_min ${yyyymmddb_min}"
    exit 1
fi
#
${date_cmd} -d "${yyyymmdde}" > /dev/null
status_date=${?}
if [ ${status_date} -ne 0 ]
then
    echo "${command} : eee : yyyymmdde ${yyyymmdde} argument invalid"
    exit 1
fi
unset status_date
#
if [[ "${yyyymmdde}" > "${yyyymmdde_max}" ]]
then
    echo "${command} : eee : yyyymmdde ${yyyymmdde} must be lower or equal to yyyymmdde_max ${yyyymmdde_max}"
    exit 1
fi
#
if [[ "${yyyymmdde}" < "${yyyymmddb}" ]]
then
    echo "${command} : eee : yyyymmdde ${yyyymmdde} must be greater than yyyymmddb ${yyyymmddb}"
    exit 1
fi
#
if [[ "${yyyymmdde}" = "${yyyymmddb}" ]]
then
     echo "${command} : eee : yyyymmdde ${yyyymmdde} must be greater than yyyymmddb ${yyyymmddb}"
     exit 1
fi
#
case ${product} in
    classif)
         prefix="ClassifAMMAp2st"
         legend="Cloud Classification"
         unite=0
         scalefactor=1
         addoffset=128.
         missing=-128
         nbytes=1
         swap=0
         plotpal="520,0 0,0"
    ;;
    pressure)
         prefix="ClassifAMMAp3sp"
         legend="Cloud Top Pressure"
         unite="hPa"
         scalefactor=25.
         addoffset=2950.
         missing=-128
         nbytes=1
         swap=0
         plotpal=0
    ;;
    height)
         prefix="ClassifAMMAp3sh"
         legend="Cloud Top Altitude"
         unite="m"
         scalefactor=200.
         addoffset=23600.
         missing=-128
         nbytes=1
         swap=0
         plotpal=0
    ;;
    temperature)
         prefix="ClassifAMMAp3st"
         legend="Cloud Top Temperature"
         unite="K"
         scalefactor=1
         addoffset=278.
         missing=-128
         nbytes=1
         swap=0
         plotpal=0
    ;;
    tb087)
         prefix="AtlSudMir087"
         legend="Brightness Temperatures (8.7 microns)"
         unite="K"
         scalefactor=0.01
         addoffset=327.68
         missing=32767
         nbytes=2
         swap=1
         plotpal=0
    ;;
    tb097)
         prefix="AtlSudMir097"
         legend="Brightness Temperatures (9.7 microns)"
         unite="K"
         scalefactor=0.01
         addoffset=327.68
         missing=32767
         nbytes=2
         swap=1
         plotpal=0
    ;;
    tb108)
         prefix="AtlSudMir108"
         legend="Brightness Temperatures (10.8 microns)"
         unite="K"
         scalefactor=0.01
         addoffset=327.68
         missing=32767
         nbytes=2
         swap=1
         plotpal=0
    ;;
    tb108z)
         prefix="AtlSudMir108"
         legend="Brightness Temperatures (10.8 microns)"
         unite="K"
         scalefactor=0.01
         addoffset=327.68
         missing=32767
         nbytes=2
         swap=1
         plotpal=0
    ;;
    tb120)
         prefix="AtlSudMir120"
         legend="Brightness Temperatures (12.07 microns)"
         unite="K"
         scalefactor=0.01
         addoffset=327.68
         missing=32767
         nbytes=2
         swap=1
         plotpal=0
    ;;
    tb134)
         prefix="AtlSudMir134"
         legend="Brightness Temperatures (13.4 microns)"
         unite="K"
         scalefactor=0.01
         addoffset=327.68
         missing=32767
         nbytes=2
         swap=1
         plotpal=0
    ;;
    *)
     echo "${command} : eee : product ${product} argument invalid"
     exit 1
    ;;
esac
#
# check for ${PROJECT_LOG} definition
if [ "${PROJECT_LOG}" = "" ]
then
   echo "${command} : eee : \${PROJECT_LOG} not defined"
   exit 1
fi
#
# check for ${PROJECT_LOG} existence
if [ ! -d ${PROJECT_LOG} ]
then
   echo "${command} : eee : ${PROJECT_LOG} not found"
   exit 1
fi
#
# check for permission access on PROJECT_LOG
if [ ! -x ${PROJECT_LOG} ]
then
   echo "${command} : eee : ${PROJECT_LOG} not reachable"
   exit 1
fi
#
# check for write permission on PROJECT_LOG
if [ ! -w ${PROJECT_LOG} ]
then
   echo "${command} : eee : ${PROJECT_LOG} not writable"
   exit 1
fi
#
log=${PROJECT_LOG}/$(basename ${0} .sh).log.${log_date}
echo "[Context]" 1>> ${log}
echo "command=$(basename ${0})" 1>>${log}
echo "hostname=${hostname}" 1>> ${log}
echo "runtime=${log_date}" 1>> ${log}
echo "log=${log}" 1>> ${log}
unset log_date
echo "" 1>> ${log}
echo "[Parameters]" 1>> ${log}
echo "product=${product}" 1>> ${log}
echo "yyyymmddb=${yyyymmddb}" 1>> ${log}
echo "yyyymmdde=${yyyymmdde}" 1>> ${log}
echo "" 1>> ${log}
#
nbscript=`ps -eaf | grep -v grep |grep ${command}|grep "/bin/sh"|wc -l`
if [ ${nbscript} -gt 2 ]; then
  echo "Un script ${command} est encore en cours d'execution"
  ps -eaf | grep -v grep |grep ${command}
  exit 1
fi
#
# loop over all dates to build URL lists
# set current and end date
list_url=""
current=$(${date_cmd} -u -d "${yyyymmddb}" +%s | awk '{printf "%d",$1}')
declare -i current
end=$(${date_cmd} -u -d "${yyyymmdde}" +%s | awk '{printf "%d",$1}')
declare -i end
declare -i delta=900
while [ ${current} -lt ${end} ]
do
    echo "iii : looking for files of $(${date_cmd} -u -d "1970-01-01 ${current} sec" +%Y%m%dT%H%M)" 1>> ${log}
    yyyymmdd=$(${date_cmd} -u -d "1970-01-01 ${current} sec" +%Y%m%d)
    yyyy=$(${date_cmd} -u -d "1970-01-01 ${current} sec" +%Y)
    mm=$(${date_cmd} -u -d "1970-01-01 ${current} sec" +%m)
    dd=$(${date_cmd} -u -d "1970-01-01 ${current} sec" +%d)
    hh=$(${date_cmd} -u -d "1970-01-01 ${current} sec" +%H)
    mn=$(${date_cmd} -u -d "1970-01-01 ${current} sec" +%M)
    hhmn=$(${date_cmd} -u -d "1970-01-01 ${current} sec" +%H%M)
    dir="${PROJECT_ID}/MSG/${yyyy}/${mm}/${dd}/"
    bfile_raw_gz=${prefix}_msg01_${yyyymmdd}_${hhmn}_?????.raw.gz
    file_raw_gz=${dir}/${bfile_raw_gz}
    file_raw="${dir}/${yyyymmdd}${hhmn}_msg-${product}_map_15min.raw"
    bfile_def_gz=${prefix}_msg01_${yyyymmdd}_${hhmn}_?????.def.gz
    file_def_gz=${dir}/${bfile_def_gz}
    file_def="${dir}/${yyyymmdd}${hhmn}_msg-${product}_map_15min.def"
    exist=0
    if [ ! -f ${file_raw_gz} ]
    then
        echo "iii : ${file_raw_gz} not found" >> ${log} 2>&1
    else
        echo "iii : ${file_raw_gz} found" >> ${log} 2>&1
        exist=$(( ${exist} + 1))
    fi
    if [ ! -f ${file_def_gz} ]
    then
        echo "iii : ${file_def_gz} not found" >> ${log} 2>&1
    else
        echo "iii : ${file_def_gz} found" >> ${log} 2>&1
        exist=$(( ${exist} + 1))
    fi
    if [ ${exist} -eq 0 ]
    then
        echo "iii : missing both of ${file_raw_gz} or ${file_def_gz}" >> ${log} 2>&1
    fi
    if [ ${exist} -eq 1 ]
    then
        echo "iii : missing one of ${file_raw_gz} or ${file_def_gz}" >> ${log} 2>&1
    fi
    if [ ${exist} -eq 2 ]
    then
        gunzip -c ${file_raw_gz} > ${file_raw}
        gunzip -c ${file_def_gz} > ${file_def}
        file_netcdf="${dir}/${yyyymmdd}${hhmn}_msg-${product}_map_15min.nc"
        file_gif="${dir}/${yyyymmdd}${hhmn}_msg-${product}_map_15min.gif"
        #
        create_nc.sh \
           ${yyyy} ${mm} ${dd} ${hh} ${mn} \
           ${file_raw} ${file_def} \
           ${file_netcdf} \
           "${legend}" "${unite}" \
           ${scalefactor} ${addoffset} ${nbytes} ${missing} ${swap} \
           ${lonmin} ${lonmax} ${latmin} ${latmax} >> ${log} 2>&1
        status=${?}
        if [ ${status} -ne 0 ]
        then
           echo "eee: failed to create ${file_netcdf} : ${status}" >> ${log} 2>&1
           exit 1
        fi
        unset status
        #
        create_gif.sh \
           ${file_netcdf} \
           ${file_gif} \
           ${product} \
           ${plotpal} >> ${log} 2>&1
        status=${?}
        if [ ${status} -ne 0 ]
        then
           echo "eee: failed to create ${file_gif} : ${status}" >> ${log} 2>&1
           exit 1
        fi
        unset status
    fi
    unset yyyymmdd
    unset yyyy
    unset mm
    unset dd
    unset hhmn
    unset dir
    # next time step
    current=$((${current} + ${delta}))
done
#
unset command
unset log
unset hostname
unset usage
#
unset date_cmd
#
# end
#++set
exit
