function [result]=write_station(lon_value, lat_value, altitude_value, code_value, station_value, region_value)

%WRITE_STATION write pluviometer station file :file:`${PROJECT_OD}/PRECIP/stations.csv`
%from station associated arrays lat, lon, name, region

%
%+
%
% ===============
% write_station.m
% ===============
%
% .. function:: write_station(lon_value, lat_value, altitude_value, code_value, station_value, region_value)
%
% DESCRIPTION
% ===========
%
%    :param lon_value: longitude array
%    :type lon_value: 1D array of floats
%    :raise lon_value: required
%
%    :param lat_value: latitude array
%    :type lat_value: 1D array of floats
%    :raise lat_value: required
%
%    :param altitude_value: altitude array
%    :type altitude_value: 1D array of floats
%    :raise altitude_value: required
%
%    :param station_value: station array
%    :type station_value: 1D cell array of strings
%    :raise station_value: required
%
%    :param region_value: region array
%    :type region_value: 1D cell array of strings
%    :raise region_value: required
%
%    :returns: 0 if ok or -1 if error
%    :rtype: integer
%
% Write in a file :file:`${PROJECT_OD}/PRECIP/stations_write_station.csv`
%
% EXAMPLES
% ========
%
% From scratch to write some simulated station data in
% :file:`${PROJECT_OD}/PRECIP/stations_write_station.csv`:
%
% see demo 1
%
% From real file :file:`${PROJECT_ID}/PRECIP/stations.csv`:
%
% see demo 2
%
% SEE ALSO
% ========
%
% :ref:`guide data station pluviometer <data_station_pluviometer>`
%
% :func:`simul_station`
% :func:`read_station`
%
% TODO
% ====
%
% check arguments (type, size, value)
%
% clean
%
% missing value
%
% EVOLUTIONS
% ==========
%
% $Id$
%
% $URL: svn+ssh://pinsard@forge.ipsl.jussieu.fr/ipsl/forge/projets/varamma/svn/trunk/src/write_station.m $
%
% - fplod 20110816T084401Z aedon.locean-ipsl.upmc.fr (Darwin)
%
%   * handle IO error
%
% - fplod 20110302T104350Z aedon.locean-ipsl.upmc.fr (Darwin)
%
%   * adaptation for octave 3.2.3 and 3.2.4 : new sprintf behaviour for cell
%
% - fplod 20110302T100637Z aedon.locean-ipsl.upmc.fr (Darwin)
%
%   * adaptation for matlab : can not use octave size_equal
%
% - fplod 20110302T093253Z aedon.locean-ipsl.upmc.fr (Darwin)
%
%   * test inconsistent dimensions of arguments
%   * moved example to demo (and fix)
%   * header fixes
%
% - fplod 20110217T173459Z aedon.locean-ipsl.upmc.fr (Darwin)
%
%   * reorganize writing to suppress blank before and after separators
%     nb : dlmwrite cannot be used because of strings/floats mixing
%
% - fplod 20110217T150320Z aedon.locean-ipsl.upmc.fr (Darwin)
%
%   * matlab (fprintf) can not write cells or strings !
%
% - fplod 20110217T120019Z aedon.locean-ipsl.upmc.fr (Darwin)
%
%   * usage of cells array of string to handle station_value and code_value
%     strings array are real nightmare (dealing with indices)
%
% - fplod 20110216T083826Z aedon.locean-ipsl.upmc.fr (Darwin)
%
%   * creation
%
%-
%
%
global application;
global application_version;
%
global PROJECT_OD;
%
result=-1;
%
usage='result=write_station(lon_value, lat_value, altitude_value, code_value, station_value, region_value)';
%
if nargin~=nargin(mfilename)
   disp(['Incorrect number of input arguments ' num2str(nargin)]);
   error(usage);
end
%
if nargout ~= nargout(mfilename)
   disp(['Incorrect number of output arguments ' num2str(nargout)]);
   error(usage);
end
%
arg_info=whos('lon_value');
if ~strcmp(arg_info.class,'double')
   disp(['Incorrect type of arg lon_value ' arg_info.class]);
   whos lon_value
   error(usage);
end
clear arg_info
%
if ~isvector(lon_value)
   disp('arg lon_value must be a vector');
   error(usage);
end
%
arg_info=whos('lat_value');
if ~strcmp(arg_info.class,'double')
   disp(['Incorrect type of arg lat_value ' arg_info.class]);
   whos lat_value
   error(usage);
end
clear arg_info
%
if ~isvector(lat_value)
   disp('arg lat_value must be a vector');
   error(usage);
end
%
arg_info=whos('altitude_value');
if ~strcmp(arg_info.class,'double')
   disp(['Incorrect type of arg altitude_value ' arg_info.class]);
   whos altitude_value
   error(usage);
end
clear arg_info
%
if ~isvector(altitude_value)
   disp('arg altitude_value must be a vector');
   error(usage);
end
%
arg_info=whos('code_value');
if ~strcmp(arg_info.class,'double')
   disp(['Incorrect type of arg code_value ' arg_info.class]);
   whos code_value
   error(usage);
end
clear arg_info
%
if ~isvector(code_value)
   disp('arg code_value must be a vector');
   error(usage);
end
%
arg_info=whos('station_value');
if ~strcmp(arg_info.class,'cell')
   disp(['Incorrect type of arg station_value ' arg_info.class]);
   whos station_value
   error(usage);
end
clear arg_info
%
% +todo+   disp('arg station_value must be a cell array of strings');
%
arg_info=whos('region_value');
if ~strcmp(arg_info.class,'cell')
   disp(['Incorrect type of arg region_value ' arg_info.class]);
   whos region_value
   error(usage);
end
clear arg_info
%
% +todo+   disp('arg region_value must be a cell array of strings');
%
% check for size of nb_lat nb_lon nb_station
nb_station=size(station_value,1);
%
switch application
  case {'matlab'}
     test_consistent=isequal(numel(lon_value), numel(lat_value), numel(code_value), numel(station_value), numel(region_value));
  case {'octave'}
     test_consistent=size_equal(lon_value, lat_value, altitude_value, code_value, station_value, region_value)
  otherwise
     error('unknown application running');
end
if (not(test_consistent))
   disp(['eee : inconsistent dimensions of lon_value, lat_value, altitude_value, code_value, station_value and region_value']);
   whos lon_value
   whos lat_value
   whos altitude_value
   whos code_value
   whos station_value
   whos region_value
   error(usage);
end
%
% build ASCII filename to be written
fullfilename=[PROJECT_OD '/PRECIP/'  'stations' '_' mfilename '.csv'];
disp(['iii : opening for writing ', fullfilename]);
[fid,message]=fopen(fullfilename,'w');
if (fid == -1)
    disp(message)
    error([ fullfilename ' can not be written'])
end
clear fullfilename
%
% write header
header=['Code,Station-Nom,Longitude,Latitude,Altitude,Region'];
format_header=['%s\n'];
fprintf(fid, format_header, header);
clear format_header
clear header
%
% write data
%
sep=',';
% convert cell to string if needed
switch application
  case {'matlab'}
     station_string = char(station_value);
     region_string = char(region_value);
  case {'octave'}
     switch application_version
        case {'3.0.2'}
           station_string = station_value;
           region_string = region_value;
        case {'3.2.3','3.2.4','3.4.0'}
           station_string = char(station_value);
           region_string = char(region_value);
        otherwise
           error(['unknown application version running ', application , application_version '\n']);
     end
  otherwise
     error('unknown application running');
end
format_code='%-6.6d';
format_station='%s';
format_lat='%-6.2f';
format_lon='%-7.2f';
format_altitude='%-6.0f';
format_region='%s';
% loop on stations to write each data line
for index_station=1:nb_station
    code_trim=strtrim(sprintf(format_code,code_value(index_station)));
    station_trim=strtrim(sprintf(format_station,station_string(index_station,:)));
    lon_trim=strtrim(sprintf(format_lon,lon_value(index_station)));
    lat_trim=strtrim(sprintf(format_lat,lat_value(index_station)));
    altitude_trim=strtrim(sprintf(format_altitude,altitude_value(index_station)));
    region_trim=strtrim(sprintf(format_region,region_string(index_station,:)));
    fprintf(fid, '%s%s%s%s%s%s%s%s%s%s%s\n' ...
                       , code_trim(:) ...
                       , sep ...
                       , station_trim(:) ...
                       , sep ...
                       , lon_trim(:) ...
                       , sep ...
                       , lat_trim(:) ...
                       , sep ...
                       , altitude_trim(:) ...
                       , sep ...
                       , region_trim(:));
end
clear format_data
clear sep
%
fclose(fid);
clear fid
%
%++whos;
result=0;
%
end

%!demo
%! % From scratch to write some simulated station data in
%! % :file:`${PROJECT_OD}/PRECIP/stations_write_station.csv`::
%! varamma_startup
%! more off
%! index_simulation = uint8(1);
%! [lon_value, lat_value, altitude_value, code_value, station_value, region_value] = simul_station(index_simulation);
%! result = write_station(lon_value, lat_value, altitude_value, code_value, station_value, region_value);

%!demo
%! % From real file :file:`${PROJECT_ID}/PRECIP/stations.csv`::
%! varamma_startup
%! more off
%! [lon_value, lat_value, altitude_value, code_value, station_value, region_value] = read_station();
%! result = write_station(lon_value, lat_value, altitude_value, code_value, station_value, region_value);
